<?php

/**
 * Implementation of xmpp_hook_xml().
 */
function xmpp_roster_xml() {
  global $ini;
  $funcs = array();

  $funcs['xmpp_request_roster'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'iq',
      'options' => array(
        'type' => 'get',
      ),
      'children' => array(
        'query' => array(
          'tag' => 'query',
          'options' => array(
            'xmlns' => 'jabber:iq:roster',
          ),
        ),
      ),
    ),
  );

  $funcs['xmpp_set_roster'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'iq',
      'options' => array(
        'type' => 'set',
      ),
      'children' => array(
        'query' => array(
          'tag' => 'query',
          'options' => array(
            'xmlns' => 'jabber:iq:roster',
          ),
          'children' => array(
            'item' => array(
              'tag' => 'item',
              'options' => array(
                'jid' => '@jid',
                'subscription' => '!',
              ),
            ),
          ),
        ),
      ),
    ),
  );

  $funcs['xmpp_remove_roster'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'iq',
      'options' => array(
        'type' => 'set',
      ),
      'children' => array(
        'query' => array(
          'tag' => 'query',
          'options' => array(
            'xmlns' => 'jabber:iq:roster',
          ),
          'children' => array(
            'item' => array(
              'tag' => 'item',
              'options' => array(
                'jid' => '@jid',
                'subscription' => 'remove',
              ),
            ),
          ),
        ),
      ),
    ),
  );
 
  return $funcs;
}

/**
 * Implementation of xmpp_hook_disco_features().
 */
function xmpp_roster_disco_features() {
  return array('jabber:iq:roster');
}

/**
 * Callback for xmpp's roster set iq stanza.
 */
function xmpp_set_roster($node, $s) {
  global $ini;

echo "ROSTER SET:";
print_r($node);
  $item = $node->children['query']->children['item'];

  if(isset($item->options['subscription']))
    return;

  $to = $item->options['jid'];
  $user = _xmpp_server_find_user($to);
  $item->options['jid'] = $to->Serialize();

  if(!$user) {
    $iq = new XmlNode('iq', 
      array(
        'type' => 'error', 
        'id' => $node->options['id'],
        'to' => $s->jid->Serialize(),
        'from' => $ini['host']
      ));

    $err = new XmlNode('error', array('type' => 'cancel'));
    $err->AddChild(new XmlNode('item-not-found'));

    $iq->AddChild($err);

    $s->Send($iq->Serialize());
    return;
  }
  
  $groups = array();
  foreach($item->children as $cid => $child) {
    if($child->name == 'group' && is_numeric($cid) && !in_array($groups, $child->data))
      $groups[] = $child->data;
  }

  xmpp_set_roster_entry($s->user->uid, $user, $groups, $item->options['name']);

  $response = new XmlNode('iq', array('type' => 'response', 'id' => $node->options['id']));
  $s->Send($response->Serialize());

  perform_roster_set($s);
}

/**
 * Callback for XMPP's roster get iq stanza.
 */
function xmpp_request_roster($node, $s) {
  global $ini;

  $s->gotroster = true;
  
  $iq = new XmlNode('iq', array('type' => 'result', 'id' => $node->options['id']));
  $iq->AddChild(roster_get($s));

  $s->Send($iq->Serialize());

  modules_event("got_roster", $s);
}

function xmpp_remove_roster($node, $s) {
  if($node->children['query']->children['item']->options['subscription'] != 'remove')
    return;
  $jid = $node->children['query']->children['item']->options['jid'];
  $node->children['query']->children['item']->options['jid'] = $jid->Serialize();
  $user = _xmpp_server_find_user($jid);

  xmpp_unsubscribe($s->user->uid, $user);
  
  perform_roster_set($s, $user);
  perform_roster_set($jid->Serialize(true));

/*  $sessions = router_session($jid);
  foreach($sessions as $sess) {
    perform_roster_set($sess, $s->user->uid);
    break; //perform_roster_set does this same iteration, so quit.
  }*/

  xmpp_send_presence_unavailable($s->jid, $jid);
  //xmpp_send_presence_unavailable($s->jid, $jid);
}

/**
 * Perform an XMPP 'roster set'
 * @param $s The session for the user.
 */
function perform_roster_set($s, $removed = 0) {
  $rosternode = roster_get($s, $removed);

  if($removed && is_numeric($removed)) {
    xmpp_remove_from_roster($s->user->uid, $removed);
  }
  
  $sessions = array();
  if(is_string($s))
    $sessions = router_session($s);
  else
    $sessions = router_session($s->jid->Serialize(true));

  foreach($sessions as $sess) {
    $iq = new XmlNode('iq', array('type' => 'set', 'id' => $sess->Id()));
    $iq->AddChild($rosternode);
    
    $iq->options['to'] = $sess->jid->Serialize();
    if($sess->gotroster)
      $sess->Send($iq->Serialize());
  }

}

/** 
 * Get the user's roster as a 'query' response.
 * @param $s The current user's session.
 * @return An XmlNode() representing the query response.
 */
function roster_get($s, $removed = 0) {
  global $ini;
  $user = NULL;
  if(is_object($s))
    $user = $s->user->uid;
  elseif(is_string($s))
    $user = _xmpp_server_find_user(new JID($s));
  elseif(is_numeric($s))
    $user = $s;

  $query = new XmlNode('query', array('xmlns' => 'jabber:iq:roster'));

  $friends = get_user_roster($user);

  foreach($friends as $fkey => $f) {
    $item = new XmlNode('item');
    $item->options['jid'] = "{$f->uname}@{$ini['host']}";
    if(!empty($f->name))
      $item->options['name'] = $f->name;
    if($f->fid == $removed) {
      $item->options['subscription'] = 'remove';
    } else {
      $item->options['subscription'] = $f->subscription;
    }
    
    if(_get_ask_state($user, $f->fid))
      $item->options['ask'] = 'subscribe';

    foreach($f->groups as $groupname) {
      $group = new XmlNode('group');
      $group->data = $groupname;
      $item->AddChild($group);
    }

    $query->AddChild($item);
  }
  return $query;
}

