<?php

function xmpp_router_xml() {
  $funcs = array();

  $funcs['router_message'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'message',
      'max_depth' => 2,
    ),
  );

  $funcs['router_presence'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'presence',
      'max_depth' => 2,
    ),
  );

  $funcs['router_iq'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'iq',
      'max_depth' => 2,
      'options' => array(
        'to' => '*',
      ),
    ),
  );

  return $funcs;
}

function xmpp_router_user_bind($s) {
  global $ini;
  
  $jid = $s->jid->Serialize();
  router_session($jid, $s);

}

function xmpp_router_end_session($s) {
  if($s->jid)
    router_session($s->jid->Serialize(), false);
}

function router_message($node, $s) {
  $to = router_session($node->options['to']);
  $node->options['from'] = $s->jid->Serialize();
  $sent = false;
  if(is_object($to)) {
    $to->Send($node->Serialize());
    $sent = true;
  } else if(is_array($to)) {
    foreach($to as $target) {
      $target->Send($node->Serialize());
      $sent = true;
    }
  }
  
  if(!$sent) {
    $iq = new XmlNode('iq', 
        array('type' => 'error', 
          'id' => $node->options['id'], 
          'to' => $s->jid->Serialize()));
    $err = new XmlNode('error', array('type' => 'cancel'));
    $err->AddChild(new XmlNode('service-unavailable'));
    $iq->AddChild($err);
    
    $s->Send($iq->Serialize());
  }
}

function router_presence($node, $s) {
  global $ini;
  
  if(isset($node->options['type']) && $node->options['type'] != 'unavailable')
    return;

  $node->options['from'] = $s->jid->Serialize();

  //If a 'to' clause is specified...
  if($node->options['to']) {
    $session = router_session($node->options['to']);
    foreach($session as $sess) {
      $node->options['to'] = $sess->jid->Serialize();
      $sess->Send($node->Serialize());
    }
    return;
  }

  //If we haven't set our presence before, then sync buddies
  if(!isset($s->presence)) {
    router_sync_presence($s);
  }

  $s->presence = $node;

  xmpp_send_to_subscribers($s, $node, 'from');
}

function router_sync_presence($s) {
  global $ini;
  $friends = get_user_subscriptions($s->user->uid);
  foreach($friends as $fid => $f) {
    if($f->subscription == 'from' || $f->subscription == 'none')
      continue;

    $fjid = "{$f->uname}@{$ini['host']}";
                        
    $sessions = router_session($fjid);

    foreach($sessions as $sess) {
      if(is_object($sess->presence)) {
        $p = $sess->presence;
        $p->options['to'] = $s->jid->Serialize();
        $p->options['from'] = $sess->jid->Serialize();
        $s->Send($p->Serialize());
      }
    }
  }
}

function router_iq($node, $s) {
  $node->options['from'] = $s->jid->Serialize();
  $sessions = router_session($node->options['to']);
  
  foreach($sessions as $sess) {
    $sess->Send($node->Serialize());
  }
}

/**
 * Utility function to send a node to all resources of a bare JID who have
 *   requested the roster.
 * WARNING: This function may reset the 'to' option on the node to an
 *   unpredictable JID string 
 * @param $jid The JID to send to
 * @param $node The node to send
 */
function _router_send_to($jid, $node, $bare = false, $all=true) {
  $sessions = router_session($jid->Serialize($all));
  foreach($sessions as $sess) {
    if($sess->gotroster) {
      $node->options['to'] = $sess->jid->Serialize($bare);
      $sess->Send($node->Serialize());
    }
  }
}

function router_session($s = NULL, $new = NULL) {
  static $sessions;
  if(is_string($s))
    $s = strtolower($s);
  if(!isset($sessions))
    $sessions = array();

  if($new == NULL) {
    if($s === false) {
      return $sessions;
    } else {
      if(isset($sessions[$s])) {
        return array($sessions[$s]);
      } else {
        $ret = array();
        
        $jid = new JID($s);
        if($jid->resource != '')
          return;
        $jid = $jid->name . '@' . $jid->domain;
        
        foreach($sessions as $key => $session) {
          if(strpos($key, $jid) === 0)
            $ret[$key] = $session;
        }
        return $ret;
      }
    }
  } else if($new === false) {
    unset($sessions[$s]);
  } else {
    $sessions[$s] = $new;
  }
}

