<?php

function xmpp_subscription_xml() {
  $funcs = array();
//Not yet functional
  $funcs['xmpp_presence_subscribe'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'presence',
      'options' => array(
        'to' => '@jid',
        'type' => 'subscribe',
      ),
    ),
  );
 
  $funcs['xmpp_presence_subscribed'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'presence',
      'options' => array(
        'to' => '@jid',
        'type' => 'subscribed',
      ),
    ),
  );

  $funcs['xmpp_presence_unsubscribe'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'presence',
      'options' => array(
        'to' => '@jid',
        'type' => 'unsubscribe',
      ),
    ),
  );

   $funcs['xmpp_presence_unsubscribed'] = array(
    'auth' => true,
    'filters' => array(
      'tag' => 'presence',
      'options' => array(
        'to' => '@jid',
        'type' => 'unsubscribed',
      ),
    ),
  );

  return $funcs;
}

/**
 * Callback for presence subscription stanza
 */
function xmpp_presence_subscribe($node, $s) {
  $jid = $node->options['to'];
  $user = _xmpp_server_find_user($jid);

  if(!$user || _is_subscribed($s->user->uid, $user)) { //User doesn't exist.
    return;
  }

//  if(!xmpp_user_is_on_roster($user, $s->user->uid))
//    xmpp_set_roster_entry($user, $s->user->uid);

  xmpp_subscribe($s->user->uid, $user);

  $node->options['from'] = $s->jid->Serialize(true);
  _router_send_to($jid, $node, true);

  perform_roster_set($s);
}

/**
 * Callback for presence subscription confirmation
 */
function xmpp_presence_subscribed($node, $s) {
  $jid = $node->options['to'];
  $user = _xmpp_server_find_user($jid);

  if(!$user) {//Fail silently
    return;
  }

  //Finalize $user's subscription to $s->user
  if(!xmpp_finalize_subscription($user, $s->user->uid)) {
    return; //Fail silently.
  }

  $node->options['from'] = $s->jid->Serialize();
  _router_send_to($jid, $node, true); //Forward to all resources

  perform_roster_set($s);
  perform_roster_set($jid->Serialize(true));

  $sessions = router_session($s->jid->Serialize(true));
  foreach($sessions as $sess) {
    if($sess->presence) {
      _router_send_to($jid, $sess->presence);
    }
  }
}

/**
 * Callback for presence subscription cancelation request
 */
function xmpp_presence_unsubscribe($node, $s) {
  $jid = $node->options['to'];
  $user = _xmpp_server_find_user($node->options['to']);
  xmpp_unsubscribe($s->user->uid, $user);

  perform_roster_set($s);
  
  $rosternode = roster_get($jid->Serialize());
  
  $sessions = router_session($node->options['to']->Serialize());
  foreach($sessions as $sess) {
    //Roster push
    $iq = new XmlNode('iq', array('type' => 'set', 'id' => $sess->Id()));
    $iq->AddChild($rosternode);
    
    $iq->options['to'] = $sess->jid->Serialize();
    if($sess->gotroster)
      $sess->Send($iq->Serialize());

    //Forward Unsubscribe stanza
    $node->options['from'] = $s->jid->Serialize();
    $node->options['to'] = $sess->jid->Serialize();
    $sess->Send($node->Serialize());
  }

    //Send 'unsubscribed'
  $un = new XmlNode('presence', array('type' => 'unsubscribed'));
  $un->options['to'] = $s->jid->Serialize();
  $un->options['from'] = $node->options['to'];
  $s->Send($un->Serialize());


  //Send unavailable
  xmpp_send_presence_unavailable($s->jid, $jid);
}

/**
 * Callback for presence unsubscription request.
 */
function xmpp_presence_unsubscribed($node, $s) {
  $jid = $node->options['to'];
  $user = _xmpp_server_find_user($jid);

  if(!$user) {
    return;
  }

  xmpp_unsubscribe($user, $s->user->uid);
  
  $node->options['from'] = $s->jid->Serialize(true);
  _router_send_to($jid, $node, true);

  $sessions = router_session($jid->Serialize());
  foreach($sessions as $sess) {
    perform_roster_set($sess);
    break;
  }

  xmpp_send_presence_unavailable($jid, $s->jid);
}

/**
 * Hook from roster module, triggered when a user retrieves their roster.
 */
function xmpp_subscription_got_roster($s) {
  global $ini;
  $user = $s->user->uid;

  $res = db_query("SELECT p.*, u.name 
    FROM {xmpp_server_presence} p, {users} u
    WHERE fid=%d
      AND state=%d
      AND u.uid=p.uid", 
    $user, XMPP_SERVER_SUBSCRIPTION_PENDING);

  while($row = db_fetch_object($res)) {
    $p = new XmlNode('presence', array('type' => 'subscribe'));
    $p->options['from'] = "{$row->name}@{$ini['host']}";
    $p->options['to'] = $s->jid->Serialize(true);

    $s->Send($p->Serialize());
  }
}

/**
 * Hook from core, triggered when a session ends.
 */
function xmpp_subscription_end_session($s) {
  if(!$s->user)
    return;
  $p = new XmlNode('presence', array('type' => 'unavailable'));
  $p->options['from'] = $s->jid->Serialize();
  xmpp_send_to_subscribers($s, $p);
}

function xmpp_send_to_subscribers($s, $node, $op = 'from') {
  global $ini;
  $friends = get_user_subscriptions($s->user->uid);
  foreach($friends as $fid => $f) {
    if($f->subscription == $op || $f->subscription == 'both') {
      $node->options['from'] = $s->jid->Serialize();
    
      $jid = new JID();
      $jid->name = $f->uname;
      $jid->host = $ini['host'];

      _router_send_to($jid, $node);
    }
  }
}

/**
 * Send an 'unavailable' presence notification from $jid to all resources
 *   associated with $fjid.
 * @param $jid The JID becoming unavailable.
 * @param $fjid The JID representing the user to send updates to.
 */
function xmpp_send_presence_unavailable($jid, $fjid) {
  $sessions = router_session($fjid->Serialize(true));
  foreach($sessions as $sess) {
    $p = new XmlNode('presence', array('type' => 'unavailable'));
    $p->options['from'] = $sess->jid->Serialize();
    _router_send_to($jid, $p, true);
  }
}

