<?php

function xmpp_not_authorized($node, $s) {
}

class XmlNode {
	public $options = array();
	public $data = "";
	public $children = array();
	public $name = "";

  function __construct($name = "", $options = array(), $children = array(), $data = "") {
    $this->name = $name;
    $this->options = $options;
    $this->children = $children;
    $this->data = $data;
  }

	function AddChild($node) {
    if(!is_object($node))
      return;
		$this->children[] = $node;
    $this->children[$node->name] = $node;
	}

  function Send($sock) {
    fwrite($sock, $this->Serialize());
  }

  function Serialize($op = NULL) {
    $ret = "";

    if($op == NULL || $op == 'open') {
      $ret = "<{$this->name}";
      foreach($this->options as $k => $v)
        $ret .= " $k=\"" . htmlentities($v) . "\"";
        
      if($op == NULL && empty($this->data) && count($this->children) == 0)
        return $ret . "/>"; //Short tag support
        
      $ret .= '>';
    }
    
    if($op == NULL || $op == 'body') {
      foreach($this->children as $k => $v) {
        if(is_numeric($k))
          $ret .= $v->Serialize();
      }

    $ret .= $this->data;
    }

    if($op == NULL || $op == 'close')
      $ret .= "</{$this->name}>";

    return $ret;
  }
}

class XmlStream {
  public $node;
  
	private $parser;
	private $init = false;

	private $depth = 1;

	private $cache = array();

	private $handlers = array();

	function __construct() {
		$this->parser = xml_parser_create();

		xml_set_object($this->parser, $this);
		xml_set_element_handler($this->parser, '_tag_open', '_tag_close');
		xml_set_character_data_handler($this->parser, '_cdata');
	}

	function __destruct() {
		xml_parser_free($this->parser);
	}

	function Parse($xml) {
		xml_parse($this->parser, $xml, false);
	}

	function RegisterHandler($op, $callback, $args = array(), $filters = array()) {
    if(!$op)
      $op = 'close';
		$this->handlers[$op][] = array('func' => $callback, 'args' => $args, 'filters' => $filters);
	}

	function _tag_open($parser, $tag, $attributes) {
		$tag = strtolower($tag);

    if($tag != 'stream:stream')
  		$this->depth++;

		$node = new XmlNode();
		foreach($attributes as $k => $v)
			$node->options[strtolower($k)] = $v;
		$node->name = $tag;
    
		$this->_set_tag($node);
    
    $this->_do_callbacks('open', $node);
	}

	function _tag_close($parser, $tag) {
    $op = 'close';
		if($this->depth <= 0)
			return;

		$tag = strtolower($tag);
	
		$node = $this->_get_tag();

		if($this->depth > 2) {
			$this->_get_tag(-1)->AddChild($node);
		}

    if($this->depth <= 1)
      modules_event('stanza', $node);

    $this->_do_callbacks('close', $node);
		unset($this->cache[$this->depth]);
		$this->depth--;

	}

	function _cdata($parser, $cdata) {
		$this->_get_tag()->data .= $cdata;
    $this->_do_callbacks('cdata', $this->_get_tag());
	}

  private function _set_tag($node) {
    $this->cache[$this->depth] = $node;
  }

	private function _get_tag($offset = 0) {
		return $this->cache[$this->depth + $offset];
	}

  private function _do_callbacks($op, $node) {
    if(!isset($this->handlers[$op]))
      return;

    foreach($this->handlers[$op] as $k => $v) {
      if(function_exists($v['func']) && $this->_is_callback_allowed($node, $v['filters'])) {
        array_unshift($v['args'], $node);
        if(call_user_func_array($v['func'], $v['args']) === false)
          break;
      }
    }
  }

  private function _is_callback_allowed($n, $p) {
    if(!is_array($p))
      return true;

    if($this->depth > $p['max_depth'] && isset($p['max_depth']))
      return false;

    print "Max depth pass\n";

    if($this->depth < $p['min_depth'] && isset($p['min_depth']))
      return false;

    print "Min depth pass\n";

    if(!fnmatch($p['tag'], $n->name) && isset($p['tag']))
      return false;

    print "Tag pass\n";

    $controlchars = array('#', '!', '?', '@');

    if(is_array($p['options'])) {
      foreach($p['options'] as $k => $v) {
        $first = substr($v,0,1);
        $rest = substr($v,1);

        if(empty($v))
          continue;
          
        if(is_object($n->options[$k])) //Already been parsed
          continue;
          
        if(in_array($first, $controlchars)) {
          switch($first) {
            case '#': //Regex
              $preg = preg_match($rest, $n->options[$k]);
              if($preg === false)
                echo t("Regex syntax error (@err) in: !regx", 
                    array('@err' => preg_last_error(), '!regx' => $v)) . "\n";

              if(!$preg)
                return false;

              break;
            case '!': //Not-this value
              if(fnmatch($rest, $n->options[$k]))
                return false;
              break;
            case '?': //This-or-nothing value
              if(!empty($n->options[$k]) && !fnmatch($rest, $n->options[$k]))
                return false;

              break;
            case '@': //Preprogrammed translated value
              if($rest == 'jid' || $rest == 'user') {
                $n->options[$k] = new JID($n->options[$k]);
                if($rest == 'user')
                  $n->options[$k] = _xmpp_server_find_user($n->options[$k]);
              }
              break;
          }
        } else {
          if(isset($v) && !isset($n->options[$k])) {
            //If we specified a value but none was given...
            return false;
          } elseif(!fnmatch($v, $n->options[$k]) && substr($v, -1) != '?') {
            //If the value did not match our pattern...
            return false;
          }
        }
      }
    }

    print "Options pass\n";

    if(is_array($p['children'])) {
      foreach($n->children as $child) {
        $allowed = false;
        foreach($p['children'] as $cdef) {
          $allowed = $this->_is_callback_allowed($child, $cdef);
          if($allowed)
            break;
        }
        if(!$allowed)
          return false;
      }
    }

//    print "Children pass\n";

    return true;
  }
}


