<?php

require_once("jid.php");
require_once("xmlstream.php");
require_once("module.php");
require_once("session.php");

define('XMPP_SERVER_WATCHDOG_TYPE', 'xmppd');

//Begin XMPPD bootstrapping
if(!file_exists('xmppd.ini')) {
  echo "Please make sure your php.ini file exists and is readable by the current user.\n";
  if(file_exists('xmppd.ini.example'))
    echo "It looks like you haven't configured the server yet. Please look at xmppd.ini.example and README.txt for more information.\n";
  die();
}

$ini = parse_ini_file('xmppd.ini', false);

if(isset($ini['memory_limit']))
  ini_set('memory_limit', $ini['memory_limit']);
else
  ini_set('memory_limt', '128M');

//Fool Drupal into thinking it's being run through the web.
$_SERVER['HTTP_HOST'] = $ini['http_host'];
$_SERVER['PHP_SELF'] = $ini['uriroot'].'/'.$_SERVER['SCRIPT_NAME'];
$_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'] = $_SERVER['PHP_SELF'];
$_SERVER['REMOTE_ADDR'] = NULL; // any values here do rather...
$_SERVER['REQUEST_METHOD'] = NULL; // ...odd things. uh huh.

if(!file_exists($ini['path']) || !is_dir($ini['path'])) {
  die("ERROR: Please set the 'path' variable in the 'Drupal' section of your xmppd.ini to the root of your Drupal installation.\n");
}

if(!file_exists($ini['modulespath']) || !is_dir($ini['modulespath'])) {
  die("ERROR: Your modules path does not exists. Check the value of 'modulespath' in your xmppd.ini.\n");
}

$modules = new ModuleManager($ini['module']);
$modules->Init($ini['modulespath']);

chdir($ini['path']);

//Bootstrap Drupal
require_once './includes/bootstrap.inc';
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);

if(!module_exists('xmpp_server')) {
  $msg = "Please make sure that the XMPP Server module is installed in the site @host.";
  $vars = array('@host' => $ini['http_host']);
  watchdog(XMPP_SERVER_WATCHDOG_TYPE, $msg, $vars, WATCHDOG_ERROR);
  echo t($msg, $vars) . "\n";
  die();
}

$cx = array('ssl' => array(
    'local_cert' => $ini['local_cert'], 
    'cafile' => $ini['cafile'],
    'capath' => $ini['capath'],
    'verify_peer' => false,
    'allow_self_signed' => true,
  ),
);
$cx = stream_context_create($cx);

$port = variable_get('xmppd_server_port', 5222);
if(!($sock = stream_socket_server("tcp://0.0.0.0:$port", $errno, $errstr, 
          STREAM_SERVER_BIND|STREAM_SERVER_LISTEN, $cx))) {
  $msg = "Could not open socket: @error";
  $vars = array('@error' => $errstr);
  echo t($msg, $vars) . "\n";
  watchdog(XMPP_SERVER_WATCHDOG_TYPE, $msg, $vars, WATCHDOG_ERROR);
  die();
}



//Bootstrapping complete. Continue with main loop.
$sessions = array();
$socks = array($sock);

$timers = array(
  5 => 0,
  10 => 0,
  15 => 0,
  30 => 0,
  60 => 0,
);

echo t("WARNING: This is a development release and is not recommended for a production environment.") . "\n";

watchdog(XMPP_SERVER_WATCHDOG_TYPE, "XMPPD is online.");
echo t("XMPPD is online.") . "\n";

variable_set('xmpp_server_running', true);
while(variable_get('xmpp_server_running', true)) { //Main loop
  $read = $socks;
  $write = $socks;
  $except = $socks;

  foreach($timers as $delay => $last) {
    if(time() > $last + $delay) {
      $timers[$delay] = time();
      modules_event("timer_$delay");
    }
  }
  
  $res = stream_select($read, $write, $except, 0);

  if(in_array($sock, $read)) {
    $thissock = stream_socket_accept($sock);


    $sessions[$thissock] = new Session($thissock);
      
    $socks[] = $thissock;

    $key = array_search($sock, $read);
    unset($read[$key]);
  }

  foreach($read as $v) {
    $session = $sessions[$v];
    $data = @fread($v, 1024);

    if($data === false || $data === "") {
      $session->Close();
      continue;
    }
    $jid = "unnamed session";
    if($session->jid)
      $jid = $session->jid->Serialize();
    $session->Data($data);
  }

  foreach($sessions as $s) {
    if($s->Closing()) {
      $key = array_search($s, $sessions);
      fclose($s->sock);
      unset($socks[$key]);
      unset($sessions[$key]);
    }
  }

  usleep(500); //Sleep for half a second to give the CPU time to breathe.
}

watchdog(XMPP_SERVER_WATCHDOG_TYPE, "XMPPD is offline.");
echo t("XMPPD is going down...") . "\n";

foreach($socks as $v) {
  fwrite($v, "</stream>");
  fclose($v);
}

echo t("Halted.") . "\n";

fclose($sock);

